<?php
require_once (__DIR__ . '/../dbutils.php');
require_once (__DIR__ . '/../commonutils.php');
require_once (__DIR__ . '/../roomtables.php');
class SRoomSync {	
        public function updateWorkload($pdo) {
                $url = CommonUtils::getConfigValue($pdo, 'sroomurl', "");
		if (is_null($url)) {
			return array("status" => "OK","msg" => "");
		} else {
			$url = trim($url);
			if ($url == "") {
				return array("status" => "OK","msg" => "");
			}
		}
		$sroomcode = trim(CommonUtils::getConfigValue($pdo, 'sroomcode', ''));
		
		if ($sroomcode == '') {
			return array("status" => "ERROR","msg" => "Showroom access code not set - stopping here for security reasons!");
		}
                $workfload = Roomtables::getWorkload($pdo);
                $transferdata = array(
                    "sroomcode" => $sroomcode,
                    "workload" => $workfload,
                    "updateworkload" => "1"
		);

		$data = json_encode($transferdata);
		$transferdataBase64 = base64_encode($data);

		return $this->sendToShowroom($url, $transferdataBase64);
        }
        public function checksroomaccess($url,$code) {
                if (is_null($url)) {
			return array("status" => "ERROR","msg" => "Keine URL angegeben");
		} else {
			$url = trim($url);
			if ($url == "") {
				return array("status" => "OK","msg" => "Keine URL angegeben");
			}
		}
                $sroomcode = trim($code);
		
		if ($sroomcode == '') {
			return array("status" => "ERROR","msg" => "Showroom access code nicht gesetzt!");
		}
                $transferdata = array(
                    "sroomcode" => $sroomcode,
                    "test" => "1"
		);

		$data = json_encode($transferdata);
		$transferdataBase64 = base64_encode($data);

		return $this->sendToShowroom($url, $transferdataBase64);
        }
        
	public function sync($pdo) {
		$url = CommonUtils::getConfigValue($pdo, 'sroomurl', "");
		if (is_null($url)) {
			return array("status" => "OK","msg" => "");
		} else {
			$url = trim($url);
			if ($url == "") {
				return array("status" => "OK","msg" => "");
			}
		}
		$sroomcode = trim(CommonUtils::getConfigValue($pdo, 'sroomcode', ''));
		
		if ($sroomcode == '') {
			return array("status" => "ERROR","msg" => "Showroom access code not set - stopping here for security reasons!");
		}
                
                $decpoint = CommonUtils::getConfigValue($pdo, 'decpoint', '.');
                $currency = CommonUtils::getConfigValue($pdo, 'currency', '');
                
                $sroomtitle = CommonUtils::getConfigValue($pdo, 'sroomtitle', '');
                $sroomimpressum = CommonUtils::getConfigValue($pdo, 'sroomimpressum', '');
                $sroomprivacy = CommonUtils::getConfigValue($pdo, 'sroomprivacy', '');
                $sroomcss = CommonUtils::getConfigValue($pdo, 'sroomcss', '');
                $sroomabout = CommonUtils::getConfigValue($pdo, 'sroomabout', '');
                $sroomnews = CommonUtils::getConfigValue($pdo, 'sroomnews', '');
                $sroomfood = CommonUtils::getConfigValue($pdo, 'sroomfood', '');
                $sroomdrinks = CommonUtils::getConfigValue($pdo, 'sroomdrinks', '');
                $sroomutilization = CommonUtils::getConfigValue($pdo, 'sroomutilization', '');
                $sroomprodview = CommonUtils::getConfigValue($pdo, 'sroomprodview', '0');
                $sroomshowworkload = CommonUtils::getConfigValue($pdo, 'sroomshowworkload', 1);
                
                $prodTypes = self::getMenu($pdo);
		$types = json_encode($prodTypes["types"]);
		$products = json_encode($prodTypes["products"]);
                $prodimages = self::getImagesForShowroomProducs($pdo);
                $version = CommonUtils::getConfigValue($pdo, 'version', '');

                $titleimg = self::getTitleImg($pdo);

                $workfload = Roomtables::getWorkload($pdo);
                
		$transferdata = array(
                    "sroomcode" => $sroomcode,
                    "decpoint" => $decpoint,
                    "currency" => $currency,
                    "sroomtitle" => $sroomtitle,
                    "sroomabout" => $sroomabout,
                    "sroomnews" => $sroomnews,
		    "sroomimpressum" => $sroomimpressum,
		    "sroomprivacy" => $sroomprivacy,
		    "sroomcss" => $sroomcss,
                    "types" => $types,
                    "products" => $products,
                    "prodimages" => $prodimages,
                    "titleimg" => $titleimg,
                    "sroomfood" => $sroomfood,
                    "sroomdrinks" => $sroomdrinks,
                    "sroomutilization" => $sroomutilization,
                    "sroomprodview" => $sroomprodview,
                    "coreversion" => $version,
                    "workload" => $workfload,
                    "sroomshowworkload" => $sroomshowworkload
		);

		$data = json_encode($transferdata);
		$transferdataBase64 = base64_encode($data);

		return $this->sendToShowroom($url, $transferdataBase64);
	}
	
	private function sendToShowroom($url, $data) {
		
		$url .= "/sync.php";
		
		$query = http_build_query(array("data" => $data));

		$opts = array(
		    'http' => array(
			'header' => 
			"Content-Type: application/x-www-form-urlencoded\r\n" .
			"Content-Length: " . strlen($query) . "\r\n" .
			"User-Agent:MyAgent/1.0\r\n",
			'method' => 'POST',
			'content' => $query
		    )
		);

		$context = stream_context_create($opts);

		$retContent = @file_get_contents($url, false, $context);

                if (!$retContent) {
                        error_log("No answer from showroom");
                }

		if (!$retContent) {
			$ret = array("status" => "ERROR","msg" => "Communication with showroom not successful!");
		} else {
                        if (gettype($retContent) == 'string') {
                                $ret = json_decode($retContent,true);
                        } else {
                                $ret = $retContent;
                        }
                }

                return $ret;
	}
        
        private static function getMenu($pdo) {
                $sql = "select P.id,T.kind as kind,P.longname,P.category as ref,P.priceA as price,COALESCE(P.unit,0) as unit,IF(P.days is not null, P.days, '0123456') as days ";
		$sql .= " from %products% P ,%prodtype% T where P.available='1' AND P.removed is null AND (P.display = 'KG'  OR P.display = 'K' OR P.display is null) ";
                $sql .= " AND P.category=T.id ";
		$sql .= " ORDER BY P.longname";
                $prods = CommonUtils::fetchSqlAll($pdo, $sql);
                
                $sql = "SELECT T.name from %prodtype% T WHERE (T.removed is null OR T.removed='0')";
                $types = CommonUtils::fetchSqlAll($pdo, $sql);
                
                return array("types" => $types,"products" => $prods);
        }
        
        private static function getTitleImg($pdo) {
                $sql = "SELECT setting from %logo% WHERE name=?";
                $r = CommonUtils::fetchSqlAll($pdo, $sql, array('sroomtitleimg'));
                if (count($r) == 0) {
                        return "-";
                } else {
                        return base64_encode($r[0]['setting']);
                }
        }
        
	private static function getImagesForShowroomProducs($pdo) {
		$sql = "SELECT P.id as prodid,COALESCE(I.imgm,'-') as imagedata ";
		$sql .= " FROM %products% P LEFT JOIN %prodimages% I ON P.prodimageid=I.id ";
		$sql .= " WHERE P.available='1' AND P.removed is null ";
		
		$allProductImgs = CommonUtils::fetchSqlAll($pdo, $sql);
		

		return $allProductImgs;
	}
}